@@ -23,9 +23,12 @@
 #include <inttypes.h>
 #include <stdbool.h>
 #include <libbase/libbase.h>
+#include <wayland-server-core.h>
+#include <xkbcommon/xkbcommon.h>
 
 /** Forward declaration: Wayland client handle. */
 typedef struct _wlclient_t wlclient_t;
+struct zxdg_toplevel_decoration_v1;
 
 #include "icon.h"
 #include "xdg_toplevel.h"
@@ -58,11 +61,27 @@ typedef struct {
     struct wl_seat            *wl_seat_ptr;
     /** The bound Toplevel Icon Manager. Will be NULL if not supported. */
     struct zwlmaker_icon_manager_v1 *icon_manager_ptr;
+    /** The bound XDG decoration manager. NULL if not supported. */
+    struct zxdg_decoration_manager_v1 *xdg_decoration_manager_ptr;
 
     /** Application ID, as a string. Or NULL, if not set. */
     const char                *app_id_ptr;
 } wlclient_attributes_t;
 
+/** Events of the client. */
+typedef struct {
+    /** A key was pressed. */
+    struct wl_signal          key;
+} wlclient_events_t;
+
+/** Key event. */
+typedef struct{
+    /** The key. */
+    xkb_keysym_t              keysym;
+    /** Wheter it was pressed (true) or released. */
+    bool                      pressed;
+} wlclient_key_event_t;
+
 /**
  * Creates a wayland client for simple buffer interactions.
  *
@@ -90,6 +109,9 @@ void wlclient_destroy(wlclient_t *wlclient_ptr);
 const wlclient_attributes_t *wlclient_attributes(
     const wlclient_t *wlclient_ptr);
 
+/** @return A pointer to @ref wlclient_t::events. */
+wlclient_events_t *wlclient_events(wlclient_t *wlclient_ptr);
+
 /**
  * Runs the client's mainloop.
  *
@@ -97,6 +119,14 @@ const wlclient_attributes_t *wlclient_attributes(
  */
 void wlclient_run(wlclient_t *wlclient_ptr);
 
+/**
+ * Requests termination of the client-s mainloop. This takes effect only once
+ * the mainloop wraps up an iteration.
+ *
+ * @param wlclient_ptr
+ */
+void wlclient_request_terminate(wlclient_t *wlclient_ptr);
+
 /**
  * Registers a timer with the client.
  *
