@@ -18,12 +18,15 @@
  * limitations under the License.
  */
 
+#include <libbase/libbase.h>
+#include <stddef.h>
+
 #include "segment_display.h"
 
 /** Unit tests. */
 const bs_test_set_t tests[] = {
-    { 1, "segment_dipslay", wlm_cairo_segment_display_test_cases },
-    { 0, NULL, NULL }
+    { 1, "segment_dipslay", wlm_cairo_segment_display_test_cases, NULL, NULL },
+    { 0, NULL, NULL, NULL, NULL }
 };
 
 #if !defined(TEST_DATA_DIR)
