@@ -20,17 +20,19 @@
 
 #include "cursor.h"
 
-#include "config.h"
-#include "toolkit/toolkit.h"
-
 #include <libbase/libbase.h>
-
+#include <inttypes.h>
+#include <stdlib.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_input_device.h>
 #include <wlr/types/wlr_xcursor_manager.h>
 #undef WLR_USE_UNSTABLE
 
+#include "config.h"
+#include "idle.h"
+#include "toolkit/toolkit.h"
+
 /* == Declarations ========================================================= */
 
 static void handle_motion(
@@ -58,7 +60,9 @@ static void process_motion(wlmaker_cursor_t *cursor_ptr, uint32_t time_msec);
 /* == Exported methods ===================================================== */
 
 /* ------------------------------------------------------------------------- */
-wlmaker_cursor_t *wlmaker_cursor_create(wlmaker_server_t *server_ptr)
+wlmaker_cursor_t *wlmaker_cursor_create(
+    wlmaker_server_t *server_ptr,
+    struct wlr_output_layout *wlr_output_layout_ptr)
 {
     wlmaker_cursor_t *cursor_ptr = logged_calloc(1, sizeof(wlmaker_cursor_t));
     if (NULL == cursor_ptr) return NULL;
@@ -72,10 +76,8 @@ wlmaker_cursor_t *wlmaker_cursor_create(wlmaker_server_t *server_ptr)
         wlmaker_cursor_destroy(cursor_ptr);
         return NULL;
     }
-    // Must be initialized after wlr_output_layout_ptr.
-    BS_ASSERT(NULL != server_ptr->wlr_output_layout_ptr);
-    wlr_cursor_attach_output_layout(cursor_ptr->wlr_cursor_ptr,
-                                    server_ptr->wlr_output_layout_ptr);
+    wlr_cursor_attach_output_layout(
+        cursor_ptr->wlr_cursor_ptr, wlr_output_layout_ptr);
 
     cursor_ptr->wlr_xcursor_manager_ptr = wlr_xcursor_manager_create(
         server_ptr->style.cursor.name_ptr,
@@ -97,6 +99,14 @@ wlmaker_cursor_t *wlmaker_cursor_create(wlmaker_server_t *server_ptr)
         return NULL;
     }
 
+    cursor_ptr->pointer_ptr = wlmtk_pointer_create(
+        cursor_ptr->wlr_cursor_ptr,
+        cursor_ptr->wlr_xcursor_manager_ptr);
+    if (NULL == cursor_ptr->pointer_ptr) {
+        wlmaker_cursor_destroy(cursor_ptr);
+        return NULL;
+    }
+
     wl_signal_init(&cursor_ptr->position_updated);
 
     // tinywl: wlr_cursor *only* displays an image on screen. It does not move
@@ -140,6 +150,11 @@ wlmaker_cursor_t *wlmaker_cursor_create(wlmaker_server_t *server_ptr)
 /* ------------------------------------------------------------------------- */
 void wlmaker_cursor_destroy(wlmaker_cursor_t *cursor_ptr)
 {
+    if (NULL != cursor_ptr->pointer_ptr) {
+        wlmtk_pointer_destroy(cursor_ptr->pointer_ptr);
+        cursor_ptr->pointer_ptr = NULL;
+    }
+
     if (NULL != cursor_ptr->wlr_xcursor_manager_ptr) {
         wlr_xcursor_manager_destroy(cursor_ptr->wlr_xcursor_manager_ptr);
         cursor_ptr->wlr_xcursor_manager_ptr = NULL;
@@ -343,7 +358,8 @@ void process_motion(wlmaker_cursor_t *cursor_ptr, uint32_t time_msec)
         cursor_ptr->server_ptr->root_ptr,
         cursor_ptr->wlr_cursor_ptr->x,
         cursor_ptr->wlr_cursor_ptr->y,
-        time_msec);
+        time_msec,
+        cursor_ptr->pointer_ptr);
 }
 
 /* == End of cursor.c ====================================================== */
