@@ -20,12 +20,25 @@
 
 #include "dock.h"
 
+#include <libbase/libbase.h>
+#include <libbase/plist.h>
+#include <stdbool.h>
+#include <stdlib.h>
+#include <wayland-server-core.h>
+#include <wayland-util.h>
+#define WLR_USE_UNSTABLE
+#include <wlr/types/wlr_output.h>
+#include <wlr/types/wlr_output_layout.h>
+#include <wlr/types/wlr_scene.h>
 #include <wlr/util/edges.h>
-#include "toolkit/toolkit.h"
+#undef WLR_USE_UNSTABLE
 
+#include "backend/backend.h"
+#include "backend/output_config.h"
 #include "config.h"
-#include "launcher.h"
 #include "default_state.h"
+#include "launcher.h"
+#include "toolkit/toolkit.h"
 
 /* == Declarations ========================================================= */
 
@@ -37,17 +50,26 @@ struct _wlmaker_dock_t {
     /** Back-link to server. */
     wlmaker_server_t          *server_ptr;
 
+    /** Description of the desired output, if any. */
+    wlmbe_output_description_t output_description;
+
     /** Listener for @ref wlmtk_root_events_t::workspace_changed. */
     struct wl_listener        workspace_changed_listener;
+    /** Listener for wlr_output_layout::events.change. */
+    struct wl_listener        output_layout_change_listener;
 };
 
 static bool _wlmaker_dock_decode_launchers(
-    wlmcfg_object_t *object_ptr,
+    bspl_object_t *object_ptr,
+    const union bspl_desc_value *desc_value_ptr,
     void *dest_ptr);
 
 static void _wlmaker_dock_handle_workspace_changed(
     struct wl_listener *listener_ptr,
     void *data_ptr);
+static void _wlmaker_dock_handle_output_layout_change(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 /* == Data ================================================================= */
 
@@ -56,27 +78,33 @@ typedef struct {
     /** Positioning data. */
     wlmtk_dock_positioning_t  positioning;
     /** Launchers. */
-    wlmcfg_array_t            *launchers_array_ptr;
+    bspl_array_t            *launchers_array_ptr;
 } parse_args;
 
 /** Enum descriptor for `enum wlr_edges`. */
-static const wlmcfg_enum_desc_t _wlmaker_dock_edges[] = {
-    WLMCFG_ENUM("TOP", WLR_EDGE_TOP),
-    WLMCFG_ENUM("BOTTOM", WLR_EDGE_BOTTOM),
-    WLMCFG_ENUM("LEFT", WLR_EDGE_LEFT),
-    WLMCFG_ENUM("RIGHT", WLR_EDGE_RIGHT),
-    WLMCFG_ENUM_SENTINEL(),
+static const bspl_enum_desc_t _wlmaker_dock_edges[] = {
+    BSPL_ENUM("TOP", WLR_EDGE_TOP),
+    BSPL_ENUM("BOTTOM", WLR_EDGE_BOTTOM),
+    BSPL_ENUM("LEFT", WLR_EDGE_LEFT),
+    BSPL_ENUM("RIGHT", WLR_EDGE_RIGHT),
+    BSPL_ENUM_SENTINEL(),
 };
 
 /** Descriptor for the dock's plist. */
-const wlmcfg_desc_t _wlmaker_dock_desc[] = {
-    WLMCFG_DESC_ENUM("Edge", true, parse_args, positioning.edge,
-                     WLR_EDGE_NONE, _wlmaker_dock_edges),
-    WLMCFG_DESC_ENUM("Anchor", true, parse_args, positioning.anchor,
-                     WLR_EDGE_NONE, _wlmaker_dock_edges),
-    WLMCFG_DESC_CUSTOM("Launchers", true, parse_args, launchers_array_ptr,
-                       _wlmaker_dock_decode_launchers, NULL, NULL),
-    WLMCFG_DESC_SENTINEL(),
+const bspl_desc_t _wlmaker_dock_desc[] = {
+    BSPL_DESC_ENUM("Edge", true, parse_args,
+                   positioning.edge, positioning.edge,
+                   WLR_EDGE_NONE, _wlmaker_dock_edges),
+    BSPL_DESC_ENUM("Anchor", true, parse_args,
+                   positioning.anchor, positioning.anchor,
+                   WLR_EDGE_NONE, _wlmaker_dock_edges),
+    BSPL_DESC_CUSTOM("Launchers", true, parse_args, launchers_array_ptr,
+                     launchers_array_ptr,
+                     _wlmaker_dock_decode_launchers,
+                     NULL,
+                     NULL,
+                     NULL),
+    BSPL_DESC_SENTINEL(),
 };
 
 /* == Exported methods ===================================================== */
@@ -84,7 +112,7 @@ const wlmcfg_desc_t _wlmaker_dock_desc[] = {
 /* ------------------------------------------------------------------------- */
 wlmaker_dock_t *wlmaker_dock_create(
     wlmaker_server_t *server_ptr,
-    wlmcfg_dict_t *state_dict_ptr,
+    bspl_dict_t *state_dict_ptr,
     const wlmaker_config_style_t *style_ptr)
 {
     wlmaker_dock_t *dock_ptr = logged_calloc(1, sizeof(wlmaker_dock_t));
@@ -92,16 +120,24 @@ wlmaker_dock_t *wlmaker_dock_create(
     dock_ptr->server_ptr = server_ptr;
 
     parse_args args = {};
-    wlmcfg_dict_t *dict_ptr = wlmcfg_dict_get_dict(state_dict_ptr, "Dock");
+    bspl_dict_t *dict_ptr = bspl_dict_get_dict(state_dict_ptr, "Dock");
     if (NULL == dict_ptr) {
         bs_log(BS_ERROR, "No 'Dock' dict found in configuration.");
         wlmaker_dock_destroy(dock_ptr);
         return NULL;
     }
-    wlmcfg_decode_dict(dict_ptr, _wlmaker_dock_desc, &args);
+    bspl_decode_dict(dict_ptr, _wlmaker_dock_desc, &args);
+    bspl_dict_t *output_dict_ptr = bspl_dict_get_dict(dict_ptr, "Output");
+    if (NULL != output_dict_ptr) {
+        if (!wlmbe_output_description_init_from_plist(
+                &dock_ptr->output_description, output_dict_ptr)) {
+            wlmaker_dock_destroy(dock_ptr);
+            return NULL;
+        }
+    }
 
     dock_ptr->wlmtk_dock_ptr = wlmtk_dock_create(
-        &args.positioning, &style_ptr->dock, server_ptr->env_ptr);
+        &args.positioning, &style_ptr->dock);
     if (NULL == dock_ptr->wlmtk_dock_ptr) {
         wlmaker_dock_destroy(dock_ptr);
         return NULL;
@@ -110,19 +146,30 @@ wlmaker_dock_t *wlmaker_dock_create(
         wlmtk_dock_element(dock_ptr->wlmtk_dock_ptr),
         true);
 
+    struct wlr_output *wlr_output_ptr = wlmbe_output_description_first_fnmatch(
+        &dock_ptr->output_description, server_ptr->wlr_output_layout_ptr);
+    if (NULL == wlr_output_ptr) {
+        wlr_output_ptr = wlmbe_primary_output(
+            server_ptr->wlr_output_layout_ptr);
+    }
+
     wlmtk_workspace_t *workspace_ptr =
         wlmtk_root_get_current_workspace(server_ptr->root_ptr);
     wlmtk_layer_t *layer_ptr = wlmtk_workspace_get_layer(
         workspace_ptr, WLMTK_WORKSPACE_LAYER_TOP);
-    wlmtk_layer_add_panel(
-        layer_ptr,
-        wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr));
+    if (!wlmtk_layer_add_panel(
+            layer_ptr,
+            wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr),
+            wlr_output_ptr)) {
+        wlmaker_dock_destroy(dock_ptr);
+        return NULL;
+    }
 
     for (size_t i = 0;
-         i < wlmcfg_array_size(args.launchers_array_ptr);
+         i < bspl_array_size(args.launchers_array_ptr);
          ++i) {
-        wlmcfg_dict_t *dict_ptr = wlmcfg_dict_from_object(
-            wlmcfg_array_at(args.launchers_array_ptr, i));
+        bspl_dict_t *dict_ptr = bspl_dict_from_object(
+            bspl_array_at(args.launchers_array_ptr, i));
         if (NULL == dict_ptr) {
             bs_log(BS_ERROR, "Elements of 'Launchers' must be dicts.");
             wlmaker_dock_destroy(dock_ptr);
@@ -130,7 +177,7 @@ wlmaker_dock_t *wlmaker_dock_create(
         }
         wlmaker_launcher_t *launcher_ptr = wlmaker_launcher_create_from_plist(
             &style_ptr->tile, dict_ptr,
-            server_ptr->monitor_ptr, server_ptr->env_ptr);
+            server_ptr->monitor_ptr);
         if (NULL == launcher_ptr) {
             wlmaker_dock_destroy(dock_ptr);
             return NULL;
@@ -141,7 +188,7 @@ wlmaker_dock_t *wlmaker_dock_create(
     }
     // FIXME: This is leaky.
     if (NULL != args.launchers_array_ptr) {
-        wlmcfg_array_unref(args.launchers_array_ptr);
+        bspl_array_unref(args.launchers_array_ptr);
         args.launchers_array_ptr = NULL;
     }
 
@@ -150,6 +197,15 @@ wlmaker_dock_t *wlmaker_dock_create(
         &dock_ptr->workspace_changed_listener,
         _wlmaker_dock_handle_workspace_changed);
 
+    // TODO(kaeser@gubbe.ch): This is a very hacky way of updating the output
+    // before the layer's handler removes all associated panels. Should be
+    // a native method of wlmtk_dock_t or wlmtk_panel_t.
+    dock_ptr->output_layout_change_listener.notify =
+        _wlmaker_dock_handle_output_layout_change;
+    wl_list_insert(
+        server_ptr->wlr_output_layout_ptr->events.change.listener_list.next,
+        &dock_ptr->output_layout_change_listener.link);
+
     bs_log(BS_INFO, "Created dock %p", dock_ptr);
     return dock_ptr;
 }
@@ -157,17 +213,22 @@ wlmaker_dock_t *wlmaker_dock_create(
 /* ------------------------------------------------------------------------- */
 void wlmaker_dock_destroy(wlmaker_dock_t *dock_ptr)
 {
+    wlmtk_util_disconnect_listener(&dock_ptr->output_layout_change_listener);
     wlmtk_util_disconnect_listener(&dock_ptr->workspace_changed_listener);
 
     if (NULL != dock_ptr->wlmtk_dock_ptr) {
-        wlmtk_layer_remove_panel(
-            wlmtk_panel_get_layer(wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr)),
-            wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr));
+        if (NULL != wlmtk_panel_get_layer(
+                wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr))) {
+            wlmtk_layer_remove_panel(
+                wlmtk_panel_get_layer(wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr)),
+                wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr));
+        }
 
         wlmtk_dock_destroy(dock_ptr->wlmtk_dock_ptr);
         dock_ptr->wlmtk_dock_ptr = NULL;
     }
 
+    wlmbe_output_description_fini(&dock_ptr->output_description);
     free(dock_ptr);
 }
 
@@ -176,15 +237,16 @@ void wlmaker_dock_destroy(wlmaker_dock_t *dock_ptr)
 /* ------------------------------------------------------------------------- */
 /** Decoder for the "Launchers" array. Currently just stores a reference. */
 bool _wlmaker_dock_decode_launchers(
-    wlmcfg_object_t *object_ptr,
+    bspl_object_t *object_ptr,
+    __UNUSED__ const union bspl_desc_value *desc_value_ptr,
     void *dest_ptr)
 {
-    wlmcfg_array_t **array_ptr_ptr = dest_ptr;
+    bspl_array_t **array_ptr_ptr = dest_ptr;
 
-    *array_ptr_ptr = wlmcfg_array_from_object(object_ptr);
+    *array_ptr_ptr = bspl_array_from_object(object_ptr);
     if (NULL == *array_ptr_ptr) return false;
 
-    wlmcfg_object_ref(wlmcfg_object_from_array(*array_ptr_ptr));
+    bspl_object_ref(bspl_object_from_array(*array_ptr_ptr));
     return true;
 }
 
@@ -209,7 +271,37 @@ void _wlmaker_dock_handle_workspace_changed(
     if (NULL != current_layer_ptr) {
         wlmtk_layer_remove_panel(current_layer_ptr, panel_ptr);
     }
-    wlmtk_layer_add_panel(new_layer_ptr, panel_ptr);
+    BS_ASSERT(wlmtk_layer_add_panel(
+                  new_layer_ptr,
+                  panel_ptr,
+                  wlmbe_primary_output(
+                      dock_ptr->server_ptr->wlr_output_layout_ptr)));
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handles when output layout changes; Re-computes the output to attach. */
+void _wlmaker_dock_handle_output_layout_change(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmaker_dock_t *dock_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmaker_dock_t, output_layout_change_listener);
+
+    struct wlr_output *wlr_output_ptr = wlmbe_output_description_first_fnmatch(
+        &dock_ptr->output_description, dock_ptr->server_ptr->wlr_output_layout_ptr);
+    if (NULL == wlr_output_ptr) {
+        wlr_output_ptr = wlmbe_primary_output(
+            dock_ptr->server_ptr->wlr_output_layout_ptr);
+    }
+    wlmtk_layer_t *layer_ptr = wlmtk_panel_get_layer(
+        wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr));
+    wlmtk_layer_remove_panel(layer_ptr, wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr));
+    if (NULL != wlr_output_ptr) {
+        BS_ASSERT(wlmtk_layer_add_panel(
+                      layer_ptr,
+                      wlmtk_dock_panel(dock_ptr->wlmtk_dock_ptr),
+                      wlr_output_ptr));
+    }
 }
 
 /* == Unit tests =========================================================== */
@@ -227,30 +319,45 @@ void test_create_destroy(bs_test_t *test_ptr)
 {
     struct wlr_scene *wlr_scene_ptr = wlr_scene_create();
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wlr_scene_ptr);
-    wlmtk_root_t *root_ptr = wlmtk_root_create(wlr_scene_ptr, NULL);
-    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_ptr);
-    wlmtk_tile_style_t ts = {};
-    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create("1", &ts, 0);
-    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
-    wlmtk_root_add_workspace(root_ptr, ws_ptr);
+    wlmaker_server_t server = {
+        .wlr_scene_ptr = wlr_scene_ptr,
+        .wl_display_ptr = wl_display_create(),
+    };
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, server.wl_display_ptr);
+    server.wlr_output_layout_ptr = wlr_output_layout_create(
+        server.wl_display_ptr);
+    struct wlr_output output = { .width = 1024, .height = 768, .scale = 1 };
+    wlmtk_test_wlr_output_init(&output);
+    wlr_output_layout_add_auto(server.wlr_output_layout_ptr, &output);
 
-    wlmaker_server_t server = { .root_ptr = root_ptr };
-    wlmaker_config_style_t style = {};
-
-    wlmcfg_dict_t *dict_ptr = wlmcfg_dict_from_object(
-        wlmcfg_create_object_from_plist_data(
+    bspl_dict_t *dict_ptr = bspl_dict_from_object(
+        bspl_create_object_from_plist_data(
             embedded_binary_default_state_data,
             embedded_binary_default_state_size));
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, dict_ptr);
 
+    server.root_ptr = wlmtk_root_create(
+        server.wlr_scene_ptr,
+        server.wlr_output_layout_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, server.root_ptr);
+
+    wlmtk_tile_style_t ts = {};
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create(
+        server.wlr_output_layout_ptr, "1", &ts);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
+    wlmtk_root_add_workspace(server.root_ptr, ws_ptr);
+
+    wlmaker_config_style_t style = {};
+
     wlmaker_dock_t *dock_ptr = wlmaker_dock_create(&server, dict_ptr, &style);
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, dock_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, dock_ptr);
 
     wlmaker_dock_destroy(dock_ptr);
-    wlmcfg_dict_unref(dict_ptr);
-    wlmtk_root_remove_workspace(root_ptr, ws_ptr);
+    bspl_dict_unref(dict_ptr);
+    wlmtk_root_remove_workspace(server.root_ptr, ws_ptr);
     wlmtk_workspace_destroy(ws_ptr);
-    wlmtk_root_destroy(root_ptr);
+    wlmtk_root_destroy(server.root_ptr);
+    wl_display_destroy(server.wl_display_ptr);
     wlr_scene_node_destroy(&wlr_scene_ptr->tree.node);
 }
 
