@@ -20,12 +20,15 @@
 
 #include "launcher.h"
 
-#include <limits.h>
+#include <cairo.h>
 #include <libbase/libbase.h>
-#include "toolkit/toolkit.h"
+#include <libbase/plist.h>
+#include <limits.h>
+#include <linux/input-event-codes.h>
+#include <stdbool.h>
+#include <stdlib.h>
 
-#include "conf/decode.h"
-#include "conf/plist.h"
+#include "toolkit/toolkit.h"
 
 /* == Declarations ========================================================= */
 
@@ -58,12 +61,12 @@ struct _wlmaker_launcher_t {
 };
 
 /** Plist descroptor for a launcher. */
-static const wlmcfg_desc_t _wlmaker_launcher_plist_desc[] = {
-    WLMCFG_DESC_STRING(
-        "CommandLine", true, wlmaker_launcher_t, cmdline_ptr, ""),
-    WLMCFG_DESC_STRING(
-        "Icon", true, wlmaker_launcher_t, icon_path_ptr, ""),
-    WLMCFG_DESC_SENTINEL(),
+static const bspl_desc_t _wlmaker_launcher_plist_desc[] = {
+    BSPL_DESC_STRING(
+        "CommandLine", true, wlmaker_launcher_t, cmdline_ptr, cmdline_ptr, ""),
+    BSPL_DESC_STRING(
+        "Icon", true, wlmaker_launcher_t, icon_path_ptr, icon_path_ptr, ""),
+    BSPL_DESC_SENTINEL(),
 };
 
 /** Lookup paths for icons. */
@@ -126,16 +129,15 @@ static const wlmtk_element_vmt_t _wlmaker_launcher_element_vmt = {
 /* ------------------------------------------------------------------------- */
 wlmaker_launcher_t *wlmaker_launcher_create_from_plist(
     const wlmtk_tile_style_t *style_ptr,
-    wlmcfg_dict_t *dict_ptr,
-    wlmaker_subprocess_monitor_t *monitor_ptr,
-    wlmtk_env_t *env_ptr)
+    bspl_dict_t *dict_ptr,
+    wlmaker_subprocess_monitor_t *monitor_ptr)
 {
     wlmaker_launcher_t *launcher_ptr = logged_calloc(
         1, sizeof(wlmaker_launcher_t));
     if (NULL == launcher_ptr) return NULL;
     launcher_ptr->monitor_ptr = monitor_ptr;
 
-     if (!wlmtk_tile_init(&launcher_ptr->super_tile, style_ptr, env_ptr)) {
+     if (!wlmtk_tile_init(&launcher_ptr->super_tile, style_ptr)) {
          return NULL;
     }
     launcher_ptr->orig_element_vmt = wlmtk_element_extend(
@@ -160,7 +162,7 @@ wlmaker_launcher_t *wlmaker_launcher_create_from_plist(
         return NULL;
     }
 
-    if (!wlmtk_buffer_init(&launcher_ptr->overlay_buffer, env_ptr)) {
+    if (!wlmtk_buffer_init(&launcher_ptr->overlay_buffer)) {
         wlmaker_launcher_destroy(launcher_ptr);
         return NULL;
     }
@@ -171,7 +173,7 @@ wlmaker_launcher_t *wlmaker_launcher_create_from_plist(
         &launcher_ptr->super_tile,
         wlmtk_buffer_element(&launcher_ptr->overlay_buffer));
 
-    if (!wlmcfg_decode_dict(
+    if (!bspl_decode_dict(
             dict_ptr, _wlmaker_launcher_plist_desc, launcher_ptr)) {
         bs_log(BS_ERROR, "Failed to create launcher from plist dict.");
         wlmaker_launcher_destroy(launcher_ptr);
@@ -193,8 +195,7 @@ wlmaker_launcher_t *wlmaker_launcher_create_from_plist(
     launcher_ptr->image_ptr = wlmtk_image_create_scaled(
         path_ptr,
         launcher_ptr->super_tile.style.content_size,
-        launcher_ptr->super_tile.style.content_size,
-        env_ptr);
+        launcher_ptr->super_tile.style.content_size);
     if (NULL == launcher_ptr->image_ptr) {
         wlmaker_launcher_destroy(launcher_ptr);
         return NULL;
@@ -393,7 +394,8 @@ void _wlmaker_launcher_start(wlmaker_launcher_t *launcher_ptr)
         _wlmaker_launcher_handle_window_created,
         _wlmaker_launcher_handle_window_mapped,
         _wlmaker_launcher_handle_window_unmapped,
-        _wlmaker_launcher_handle_window_destroyed);
+        _wlmaker_launcher_handle_window_destroyed,
+        NULL);
 
     if (!bs_ptr_set_insert(launcher_ptr->subprocesses_ptr,
                            subprocess_handle_ptr)) {
@@ -562,12 +564,12 @@ void test_create_from_plist(bs_test_t *test_ptr)
     static const char *plist_ptr =
         "{CommandLine = \"a\"; Icon = \"chrome-48x48.png\";}";
 
-    wlmcfg_dict_t *dict_ptr = wlmcfg_dict_from_object(
-        wlmcfg_create_object_from_plist_string(plist_ptr));
+    bspl_dict_t *dict_ptr = bspl_dict_from_object(
+        bspl_create_object_from_plist_string(plist_ptr));
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, dict_ptr);
     wlmaker_launcher_t *launcher_ptr = wlmaker_launcher_create_from_plist(
-        &style, dict_ptr, NULL, NULL);
-    wlmcfg_dict_unref(dict_ptr);
+        &style, dict_ptr, NULL);
+    bspl_dict_unref(dict_ptr);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, launcher_ptr);
 
     BS_TEST_VERIFY_STREQ(test_ptr, "a", launcher_ptr->cmdline_ptr);
