@@ -20,17 +20,26 @@
 
 #include "server.h"
 
-#include "config.h"
-#include "output.h"
-#include "toolkit/toolkit.h"
-
+#include <inttypes.h>
 #include <libbase/libbase.h>
-
-#include <wlr/version.h>
+#include <libbase/plist.h>
+#include <linux/input-event-codes.h>
+#include <stdlib.h>
+#include <wayland-server-protocol.h>
+#include <wayland-util.h>
+#include <xkbcommon/xkbcommon-keysyms.h>
 #define WLR_USE_UNSTABLE
+#include <wlr/backend.h>
 #include <wlr/types/wlr_cursor.h>
+#include <wlr/types/wlr_data_device.h>
+#include <wlr/types/wlr_input_device.h>
+#include <wlr/types/wlr_output_layout.h>
+#include <wlr/types/wlr_scene.h>
 #undef WLR_USE_UNSTABLE
 
+#include "keyboard.h"
+#include "toolkit/toolkit.h"
+
 /* == Declarations ========================================================= */
 
 /** Name of the "seat". */
@@ -65,18 +74,12 @@ static bool register_input_device(
     struct wlr_input_device *wlr_input_device_ptr,
     void *handle_ptr);
 
-static void handle_new_output(
-    struct wl_listener *listener_ptr,
-    void *data_ptr);
 static void handle_new_input_device(
     struct wl_listener *listener_ptr,
     void *data_ptr);
 static void handle_destroy_input_device(
     struct wl_listener *listener_ptr,
     void *data_ptr);
-static void handle_output_layout_change(
-    struct wl_listener *listener_ptr,
-    void *data_ptr);
 
 static void _wlmaker_server_unclaimed_button_event_handler(
     struct wl_listener *listener_ptr,
@@ -98,14 +101,14 @@ const uint32_t wlmaker_modifier_default_mask = (
 
 /* ------------------------------------------------------------------------- */
 wlmaker_server_t *wlmaker_server_create(
-    wlmcfg_dict_t *config_dict_ptr,
+    bspl_dict_t *config_dict_ptr,
     const wlmaker_server_options_t *options_ptr)
 {
     wlmaker_server_t *server_ptr = logged_calloc(1, sizeof(wlmaker_server_t));
     if (NULL == server_ptr) return NULL;
     server_ptr->options_ptr = options_ptr;
 
-    server_ptr->config_dict_ptr = wlmcfg_dict_ref(config_dict_ptr);
+    server_ptr->config_dict_ptr = bspl_dict_ref(config_dict_ptr);
     if (NULL == server_ptr->config_dict_ptr) {
         wlmaker_server_destroy(server_ptr);
         return NULL;
@@ -116,7 +119,6 @@ wlmaker_server_t *wlmaker_server_create(
 
     wl_signal_init(&server_ptr->window_created_event);
     wl_signal_init(&server_ptr->window_destroyed_event);
-    wl_signal_init(&server_ptr->output_layout_changed_event);
 
     // Prepare display and socket.
     server_ptr->wl_display_ptr = wl_display_create();
@@ -142,70 +144,6 @@ wlmaker_server_t *wlmaker_server_create(
         wlmaker_server_destroy(server_ptr);
     }
 
-    // Auto-create the wlroots backend. Can be X11 or direct.
-    server_ptr->wlr_backend_ptr = wlr_backend_autocreate(
-#if WLR_VERSION_NUM >= (18 << 8)
-        wl_display_get_event_loop(server_ptr->wl_display_ptr),
-#else // WLR_VERSION_NUM >= (18 << 8)
-        server_ptr->wl_display_ptr,
-#endif // WLR_VERSION_NUM >= (18 << 8)
-        &server_ptr->wlr_session_ptr);
-    if (NULL == server_ptr->wlr_backend_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_backend_autocreate()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-
-    // Listen for new (or newly recognized) output and input devices.
-    wlmtk_util_connect_listener_signal(
-        &server_ptr->wlr_backend_ptr->events.new_output,
-        &server_ptr->backend_new_output_listener,
-        handle_new_output);
-    wlmtk_util_connect_listener_signal(
-        &server_ptr->wlr_backend_ptr->events.new_input,
-        &server_ptr->backend_new_input_device_listener,
-        handle_new_input_device);
-
-    // Auto-create a renderer. Can be specified using WLR_RENDERER env var.
-    server_ptr->wlr_renderer_ptr = wlr_renderer_autocreate(
-        server_ptr->wlr_backend_ptr);
-    if (NULL == server_ptr->wlr_renderer_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_renderer_autocreate()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-    if (!wlr_renderer_init_wl_display(
-            server_ptr->wlr_renderer_ptr, server_ptr->wl_display_ptr)) {
-        bs_log(BS_ERROR, "Failed wlr_render_init_wl_display()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-
-    // Auto-create allocator, suitable to backend and renderer.
-    server_ptr->wlr_allocator_ptr = wlr_allocator_autocreate(
-        server_ptr->wlr_backend_ptr, server_ptr->wlr_renderer_ptr);
-    if (NULL == server_ptr->wlr_allocator_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_allocator_autocreate()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-
-    // The output layout.
-    server_ptr->wlr_output_layout_ptr = wlr_output_layout_create(
-#if WLR_VERSION_NUM >= (18 << 8)
-        server_ptr->wl_display_ptr
-#endif // WLR_VERSION_NUM >= (18 << 8)
-        );
-    if (NULL == server_ptr->wlr_output_layout_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_output_layout_create()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-    wlmtk_util_connect_listener_signal(
-        &server_ptr->wlr_output_layout_ptr->events.change,
-        &server_ptr->output_layout_change_listener,
-        handle_output_layout_change);
-
     // The scene graph.
     server_ptr->wlr_scene_ptr = wlr_scene_create();
     if (NULL == server_ptr->wlr_scene_ptr) {
@@ -213,35 +151,47 @@ wlmaker_server_t *wlmaker_server_create(
         wlmaker_server_destroy(server_ptr);
         return NULL;
     }
-    server_ptr->wlr_scene_output_layout_ptr = wlr_scene_attach_output_layout(
-        server_ptr->wlr_scene_ptr,
-        server_ptr->wlr_output_layout_ptr);
-    if (NULL == server_ptr->wlr_scene_output_layout_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_scene_attach_output_layout()");
+
+    server_ptr->wlr_output_layout_ptr = wlr_output_layout_create(
+        server_ptr->wl_display_ptr);
+    if (NULL == server_ptr->wlr_output_layout_ptr) {
+        bs_log(BS_ERROR, "Failed wlr_output_layout_create(%p)",
+               server_ptr->wl_display_ptr);
         wlmaker_server_destroy(server_ptr);
         return NULL;
     }
 
-    server_ptr->cursor_ptr = wlmaker_cursor_create(server_ptr);
+    server_ptr->backend_ptr = wlmbe_backend_create(
+        server_ptr->wl_display_ptr,
+        server_ptr->wlr_scene_ptr,
+        server_ptr->wlr_output_layout_ptr,
+        server_ptr->options_ptr->width,
+        server_ptr->options_ptr->height,
+        server_ptr->config_dict_ptr);
+    if (NULL == server_ptr->backend_ptr) {
+        bs_log(BS_ERROR, "Failed wlmbe_backend_create()");
+        wlmaker_server_destroy(server_ptr);
+        return NULL;
+    }
+    // Listen for new (or newly recognized) output and input devices.
+    wlmtk_util_connect_listener_signal(
+        &wlmbe_backend_wlr(server_ptr->backend_ptr)->events.new_input,
+        &server_ptr->backend_new_input_device_listener,
+        handle_new_input_device);
+
+    server_ptr->cursor_ptr = wlmaker_cursor_create(
+        server_ptr,
+        server_ptr->wlr_output_layout_ptr);
     if (NULL == server_ptr->cursor_ptr) {
         bs_log(BS_ERROR, "Failed wlmaker_cursor_create()");
         wlmaker_server_destroy(server_ptr);
         return NULL;
     }
 
-    server_ptr->env_ptr = wlmtk_env_create(
-        server_ptr->cursor_ptr->wlr_cursor_ptr,
-        server_ptr->cursor_ptr->wlr_xcursor_manager_ptr,
-        server_ptr->wlr_seat_ptr);
-    if (NULL == server_ptr->env_ptr) {
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-
     // Root element.
     server_ptr->root_ptr = wlmtk_root_create(
         server_ptr->wlr_scene_ptr,
-        server_ptr->env_ptr);
+        server_ptr->wlr_output_layout_ptr);
     if (NULL == server_ptr->root_ptr) {
         wlmaker_server_destroy(server_ptr);
         return NULL;
@@ -269,20 +219,6 @@ wlmaker_server_t *wlmaker_server_create(
     // The below helpers all setup a listener |display_destroy| for freeing the
     // assets held via the respective create() calls. Hence no need to call a
     // clean-up method from our end.
-    server_ptr->wlr_compositor_ptr = wlr_compositor_create(
-        server_ptr->wl_display_ptr, 5, server_ptr->wlr_renderer_ptr);
-    if (NULL == server_ptr->wlr_compositor_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_compositor_create()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
-    server_ptr->wlr_subcompositor_ptr = wlr_subcompositor_create(
-        server_ptr->wl_display_ptr);
-    if (NULL == server_ptr->wlr_subcompositor_ptr) {
-        bs_log(BS_ERROR, "Failed wlr_subcompositor_create()");
-        wlmaker_server_destroy(server_ptr);
-        return NULL;
-    }
     server_ptr->wlr_data_device_manager_ptr = wlr_data_device_manager_create(
         server_ptr->wl_display_ptr);
     if (NULL == server_ptr->wlr_data_device_manager_ptr) {
@@ -336,14 +272,14 @@ wlmaker_server_t *wlmaker_server_create(
     }
 
     server_ptr->corner_ptr = wlmaker_corner_create(
-        wlmcfg_dict_get_dict(server_ptr->config_dict_ptr, "HotCorner"),
+        bspl_dict_get_dict(server_ptr->config_dict_ptr, "HotCorner"),
         wl_display_get_event_loop(server_ptr->wl_display_ptr),
         server_ptr->wlr_output_layout_ptr,
         server_ptr->cursor_ptr,
         server_ptr);
     if (NULL == server_ptr->corner_ptr) {
         bs_log(BS_ERROR, "Failed wlmaker_corner_create(%p, %p, %p, %p, %p)",
-               wlmcfg_dict_get_dict(server_ptr->config_dict_ptr, "HotCorner"),
+               bspl_dict_get_dict(server_ptr->config_dict_ptr, "HotCorner"),
                wl_display_get_event_loop(server_ptr->wl_display_ptr),
                server_ptr->wlr_output_layout_ptr,
                server_ptr->cursor_ptr,
@@ -437,24 +373,14 @@ void wlmaker_server_destroy(wlmaker_server_t *server_ptr)
         server_ptr->root_ptr = NULL;
     }
 
-    if (NULL != server_ptr->env_ptr) {
-        wlmtk_env_destroy(server_ptr->env_ptr);
-        server_ptr->env_ptr = NULL;
-    }
-
     if (NULL != server_ptr->cursor_ptr) {
         wlmaker_cursor_destroy(server_ptr->cursor_ptr);
         server_ptr->cursor_ptr = NULL;
     }
 
-    if (NULL != server_ptr->wlr_output_layout_ptr) {
-        wlr_output_layout_destroy(server_ptr->wlr_output_layout_ptr);
-        server_ptr->wlr_output_layout_ptr = NULL;
-    }
-
-    if (NULL != server_ptr->wlr_renderer_ptr) {
-        wlr_renderer_destroy(server_ptr->wlr_renderer_ptr);
-        server_ptr->wlr_renderer_ptr = NULL;
+    if (NULL != server_ptr->backend_ptr) {
+        wlmbe_backend_destroy(server_ptr->backend_ptr);
+        server_ptr->backend_ptr = NULL;
     }
 
     if (NULL != server_ptr->wl_display_ptr) {
@@ -462,49 +388,14 @@ void wlmaker_server_destroy(wlmaker_server_t *server_ptr)
         server_ptr->wl_display_ptr = NULL;
     }
 
-    if (NULL != server_ptr->wlr_allocator_ptr) {
-        wlr_allocator_destroy(server_ptr->wlr_allocator_ptr);
-        server_ptr->wlr_allocator_ptr = NULL;
-    }
-
     if (NULL != server_ptr->config_dict_ptr) {
-        wlmcfg_dict_unref(server_ptr->config_dict_ptr);
+        bspl_dict_unref(server_ptr->config_dict_ptr);
         server_ptr->config_dict_ptr = NULL;
     }
 
     free(server_ptr);
 }
 
-/* ------------------------------------------------------------------------- */
-void wlmaker_server_output_add(wlmaker_server_t *server_ptr,
-                               wlmaker_output_t *output_ptr)
-{
-    // tinywl: Adds this to the output layout. The add_auto function arranges
-    // outputs from left-to-right in the order they appear. A sophisticated
-    // compositor would let the user configure the arrangement of outputs in
-    // the layout.
-    struct wlr_output_layout_output *wlr_output_layout_output_ptr =
-        wlr_output_layout_add_auto(server_ptr->wlr_output_layout_ptr,
-                                   output_ptr->wlr_output_ptr);
-    struct wlr_scene_output *wlr_scene_output_ptr =
-        wlr_scene_output_create(server_ptr->wlr_scene_ptr,
-                                output_ptr->wlr_output_ptr);
-    wlr_scene_output_layout_add_output(
-        server_ptr->wlr_scene_output_layout_ptr,
-        wlr_output_layout_output_ptr,
-        wlr_scene_output_ptr);
-    bs_dllist_push_back(&server_ptr->outputs, &output_ptr->node);
-}
-
-/* ------------------------------------------------------------------------- */
-void wlmaker_server_output_remove(wlmaker_server_t *server_ptr,
-                                  wlmaker_output_t *output_ptr)
-{
-    bs_dllist_remove(&server_ptr->outputs, &output_ptr->node);
-    wlr_output_layout_remove(server_ptr->wlr_output_layout_ptr,
-                             output_ptr->wlr_output_ptr);
-}
-
 /* ------------------------------------------------------------------------- */
 void wlmaker_server_activate_task_list(wlmaker_server_t *server_ptr)
 {
@@ -634,36 +525,6 @@ bool register_input_device(wlmaker_server_t *server_ptr,
     return true;
 }
 
-/* ------------------------------------------------------------------------- */
-/** Handler for the `new_output` signal raised by `wlr_backend`.
- *
- * @param listener_ptr
- * @param data_ptr
- */
-void handle_new_output(struct wl_listener *listener_ptr, void *data_ptr)
-{
-    struct wlr_output *wlr_output_ptr = data_ptr;
-    wlmaker_server_t *server_ptr = BS_CONTAINER_OF(
-        listener_ptr, wlmaker_server_t, backend_new_output_listener);
-
-    wlmaker_output_t *output_ptr = wlmaker_output_create(
-        wlr_output_ptr,
-        server_ptr->wlr_allocator_ptr,
-        server_ptr->wlr_renderer_ptr,
-        server_ptr->wlr_scene_ptr,
-        server_ptr->options_ptr->width,
-        server_ptr->options_ptr->height,
-        server_ptr);
-    if (NULL == output_ptr) {
-        bs_log(BS_INFO, "Failed wlmaker_output_create for server %p",
-               server_ptr);
-        return;
-    }
-
-    wlmaker_server_output_add(server_ptr, output_ptr);
-    bs_log(BS_INFO, "Server %p: Added output %p", server_ptr, output_ptr);
-}
-
 /* ------------------------------------------------------------------------- */
 /** Handler for the `new_input` signal raised by `wlr_backend`.
  *
@@ -747,40 +608,6 @@ void handle_destroy_input_device(struct wl_listener *listener_ptr,
     free(input_device_ptr);
 }
 
-/* ------------------------------------------------------------------------- */
-/**
- * Signal handler for `change` event of `wlr_output_layout`.
- *
- * Is emitted whenever the output layout changes. For us, this means each
- * workspace should consider re-arranging views suitably.
- *
- * @param listener_ptr
- * @param data_ptr            Points to a `struct wlr_output_layout`.
- */
-void handle_output_layout_change(
-    struct wl_listener *listener_ptr,
-    void *data_ptr)
-{
-    wlmaker_server_t *server_ptr = BS_CONTAINER_OF(
-        listener_ptr, wlmaker_server_t, output_layout_change_listener);
-    struct wlr_output_layout *wlr_output_layout_ptr = data_ptr;
-    if (wlr_output_layout_ptr != server_ptr->wlr_output_layout_ptr) {
-        // OK, this is unexpected...
-        bs_log(BS_ERROR, "Unexpected output layer mismatch: %p vs %p",
-               wlr_output_layout_ptr, server_ptr->wlr_output_layout_ptr);
-        return;
-    }
-
-    struct wlr_box extents;
-    wlr_output_layout_get_box(wlr_output_layout_ptr, NULL, &extents);
-    bs_log(BS_INFO, "Output layout change: Pos %d, %d (%d x %d).",
-           extents.x, extents.y, extents.width, extents.height);
-    wlmtk_root_set_extents(server_ptr->root_ptr, &extents);
-
-    wl_signal_emit_mutable(&server_ptr->output_layout_changed_event,
-                           &extents);
-}
-
 /* ------------------------------------------------------------------------- */
 /** Handles unclaimed button events: Right 'down' opens root menu. */
 void _wlmaker_server_unclaimed_button_event_handler(
