@@ -20,14 +20,17 @@
 
 #include "bordered.h"
 
+#include <string.h>
+
+#include "libbase/libbase.h"
+
 /* == Declarations ========================================================= */
 
-static void _wlmtk_bordered_container_update_layout(
+static bool _wlmtk_bordered_container_update_layout(
     wlmtk_container_t *container_ptr);
 
 static wlmtk_rectangle_t * _wlmtk_bordered_create_border_rectangle(
-    wlmtk_bordered_t *bordered_ptr,
-    wlmtk_env_t *env_ptr);
+    wlmtk_bordered_t *bordered_ptr);
 static void _wlmtk_bordered_destroy_border_rectangle(
     wlmtk_bordered_t *bordered_ptr,
     wlmtk_rectangle_t **rectangle_ptr_ptr);
@@ -44,31 +47,29 @@ static const wlmtk_container_vmt_t bordered_container_vmt = {
 
 /* ------------------------------------------------------------------------- */
 bool wlmtk_bordered_init(wlmtk_bordered_t *bordered_ptr,
-                         wlmtk_env_t *env_ptr,
                          wlmtk_element_t *element_ptr,
                          const wlmtk_margin_style_t *style_ptr)
 {
     BS_ASSERT(NULL != bordered_ptr);
-    memset(bordered_ptr, 0, sizeof(wlmtk_bordered_t));
-    if (!wlmtk_container_init(&bordered_ptr->super_container, env_ptr)) {
+    *bordered_ptr = (wlmtk_bordered_t){ .style = *style_ptr };
+    if (!wlmtk_container_init(&bordered_ptr->super_container)) {
         return false;
     }
     bordered_ptr->orig_super_container_vmt = wlmtk_container_extend(
         &bordered_ptr->super_container, &bordered_container_vmt);
-    memcpy(&bordered_ptr->style, style_ptr, sizeof(wlmtk_margin_style_t));
 
     bordered_ptr->element_ptr = element_ptr;
     wlmtk_container_add_element(&bordered_ptr->super_container,
                                 bordered_ptr->element_ptr);
 
     bordered_ptr->northern_border_rectangle_ptr =
-        _wlmtk_bordered_create_border_rectangle(bordered_ptr, env_ptr);
+        _wlmtk_bordered_create_border_rectangle(bordered_ptr);
     bordered_ptr->eastern_border_rectangle_ptr =
-        _wlmtk_bordered_create_border_rectangle(bordered_ptr, env_ptr);
+        _wlmtk_bordered_create_border_rectangle(bordered_ptr);
     bordered_ptr->southern_border_rectangle_ptr =
-        _wlmtk_bordered_create_border_rectangle(bordered_ptr, env_ptr);
+        _wlmtk_bordered_create_border_rectangle(bordered_ptr);
     bordered_ptr->western_border_rectangle_ptr =
-        _wlmtk_bordered_create_border_rectangle(bordered_ptr, env_ptr);
+        _wlmtk_bordered_create_border_rectangle(bordered_ptr);
     if (NULL == bordered_ptr->northern_border_rectangle_ptr ||
         NULL == bordered_ptr->eastern_border_rectangle_ptr ||
         NULL == bordered_ptr->southern_border_rectangle_ptr ||
@@ -96,14 +97,14 @@ void wlmtk_bordered_fini(wlmtk_bordered_t *bordered_ptr)
     wlmtk_container_remove_element(&bordered_ptr->super_container,
                                    bordered_ptr->element_ptr);
     wlmtk_container_fini(&bordered_ptr->super_container);
-    memset(bordered_ptr, 0, sizeof(wlmtk_bordered_t));
+    *bordered_ptr = (wlmtk_bordered_t){};
 }
 
 /* ------------------------------------------------------------------------- */
 void wlmtk_bordered_set_style(wlmtk_bordered_t *bordered_ptr,
                               const wlmtk_margin_style_t *style_ptr)
 {
-    memcpy(&bordered_ptr->style, style_ptr, sizeof(wlmtk_margin_style_t));
+    bordered_ptr->style = *style_ptr;
 
     _wlmtk_bordered_container_update_layout(&bordered_ptr->super_container);
 
@@ -134,25 +135,23 @@ wlmtk_element_t *wlmtk_bordered_element(wlmtk_bordered_t *bordered_ptr)
  *
  * @param container_ptr
  */
-void _wlmtk_bordered_container_update_layout(
+bool _wlmtk_bordered_container_update_layout(
     wlmtk_container_t *container_ptr)
 {
     wlmtk_bordered_t *bordered_ptr = BS_CONTAINER_OF(
         container_ptr, wlmtk_bordered_t, super_container);
 
     _wlmtk_bordered_set_positions(bordered_ptr);
-
-    bordered_ptr->orig_super_container_vmt.update_layout(container_ptr);
+    return true;
 }
 
 /* ------------------------------------------------------------------------- */
 /** Creates a border rectangle and adds it to `bordered_ptr`. */
 wlmtk_rectangle_t * _wlmtk_bordered_create_border_rectangle(
-    wlmtk_bordered_t *bordered_ptr,
-    wlmtk_env_t *env_ptr)
+    wlmtk_bordered_t *bordered_ptr)
 {
     wlmtk_rectangle_t *rectangle_ptr = wlmtk_rectangle_create(
-        env_ptr, 0, 0, bordered_ptr->style.color);
+        0, 0, bordered_ptr->style.color);
     if (NULL == rectangle_ptr) return NULL;
 
     wlmtk_element_set_visible(wlmtk_rectangle_element(rectangle_ptr), true);
@@ -276,7 +275,7 @@ void test_init_fini(bs_test_t *test_ptr)
 
     wlmtk_bordered_t bordered;
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_bordered_init(
-                            &bordered, NULL, &fe_ptr->element, &test_style));
+                            &bordered, &fe_ptr->element, &test_style));
 
     // Positions of border elements.
     test_rectangle_pos(
@@ -295,7 +294,7 @@ void test_init_fini(bs_test_t *test_ptr)
     // Update layout, test updated positions.
     fe_ptr->dimensions.width = 200;
     fe_ptr->dimensions.height = 120;
-    wlmtk_container_update_layout(&bordered.super_container);
+    wlmtk_container_update_layout_and_pointer_focus(&bordered.super_container);
     test_rectangle_pos(
         test_ptr, bordered.northern_border_rectangle_ptr,
         0, 0, 204, 2);
