@@ -20,20 +20,23 @@
 
 #include "resizebar_area.h"
 
-#include "box.h"
-#include "buffer.h"
-#include "gfxbuf.h"
-#include "primitives.h"
-#include "window.h"
-
+#include <cairo.h>
+#include <inttypes.h>
 #include <libbase/libbase.h>
-
+#include <linux/input-event-codes.h>
+#include <stdlib.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/interfaces/wlr_buffer.h>
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/util/edges.h>
 #undef WLR_USE_UNSTABLE
 
+#include "buffer.h"
+#include "gfxbuf.h"  // IWYU pragma: keep
+#include "input.h"
+#include "primitives.h"
+#include "window.h"
+
 /* == Declarations ========================================================= */
 
 /** State of an element of the resize bar. */
@@ -55,17 +58,10 @@ struct _wlmtk_resizebar_area_t {
     wlmtk_window_t            *window_ptr;
     /** Edges that the resizebar area controls. */
     uint32_t                  edges;
-
-    /** The cursor to use when having pointer focus. */
-    wlmtk_env_cursor_t         cursor;
 };
 
 static void _wlmtk_resizebar_area_element_destroy(
     wlmtk_element_t *element_ptr);
-static bool _wlmtk_resizebar_area_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec);
 static bool _wlmtk_resizebar_area_element_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
@@ -83,7 +79,6 @@ static struct wlr_buffer *create_buffer(
 /** Buffer implementation for title of the title bar. */
 static const wlmtk_element_vmt_t resizebar_area_element_vmt = {
     .destroy = _wlmtk_resizebar_area_element_destroy,
-    .pointer_motion = _wlmtk_resizebar_area_element_pointer_motion,
     .pointer_button = _wlmtk_resizebar_area_element_pointer_button,
 };
 
@@ -92,7 +87,6 @@ static const wlmtk_element_vmt_t resizebar_area_element_vmt = {
 /* ------------------------------------------------------------------------- */
 wlmtk_resizebar_area_t *wlmtk_resizebar_area_create(
     wlmtk_window_t *window_ptr,
-    wlmtk_env_t *env_ptr,
     uint32_t edges)
 {
     wlmtk_resizebar_area_t *resizebar_area_ptr = logged_calloc(
@@ -102,28 +96,29 @@ wlmtk_resizebar_area_t *wlmtk_resizebar_area_create(
     resizebar_area_ptr->window_ptr = window_ptr;
     resizebar_area_ptr->edges = edges;
 
-    resizebar_area_ptr->cursor = WLMTK_CURSOR_DEFAULT;
+    wlmtk_pointer_cursor_t cursor = WLMTK_POINTER_CURSOR_DEFAULT;
     switch (resizebar_area_ptr->edges) {
     case WLR_EDGE_BOTTOM:
-        resizebar_area_ptr->cursor = WLMTK_CURSOR_RESIZE_S;
+        cursor = WLMTK_POINTER_CURSOR_RESIZE_S;
         break;
     case WLR_EDGE_BOTTOM | WLR_EDGE_LEFT:
-        resizebar_area_ptr->cursor = WLMTK_CURSOR_RESIZE_SW;
+        cursor = WLMTK_POINTER_CURSOR_RESIZE_SW;
         break;
     case WLR_EDGE_BOTTOM | WLR_EDGE_RIGHT:
-        resizebar_area_ptr->cursor = WLMTK_CURSOR_RESIZE_SE;
+        cursor = WLMTK_POINTER_CURSOR_RESIZE_SE;
         break;
     default:
         bs_log(BS_ERROR, "Unsupported edge %"PRIx32, edges);
     }
 
-    if (!wlmtk_buffer_init(&resizebar_area_ptr->super_buffer, env_ptr)) {
+    if (!wlmtk_buffer_init(&resizebar_area_ptr->super_buffer)) {
         wlmtk_resizebar_area_destroy(resizebar_area_ptr);
         return NULL;
     }
     resizebar_area_ptr->orig_super_element_vmt = wlmtk_element_extend(
         &resizebar_area_ptr->super_buffer.super_element,
         &resizebar_area_element_vmt);
+    resizebar_area_ptr->super_buffer.pointer_cursor = cursor;
 
     draw_state(resizebar_area_ptr);
     return resizebar_area_ptr;
@@ -191,23 +186,6 @@ void _wlmtk_resizebar_area_element_destroy(wlmtk_element_t *element_ptr)
     wlmtk_resizebar_area_destroy(resizebar_area_ptr);
 }
 
-/* ------------------------------------------------------------------------- */
-/** See @ref wlmtk_element_vmt_t::pointer_motion. */
-bool _wlmtk_resizebar_area_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec)
-{
-    wlmtk_resizebar_area_t *resizebar_area_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_resizebar_area_t, super_buffer.super_element);
-    resizebar_area_ptr->orig_super_element_vmt.pointer_motion(
-        element_ptr, x, y, time_msec);
-
-    wlmtk_env_set_cursor(element_ptr->env_ptr, resizebar_area_ptr->cursor);
-    return true;
-}
-
 /* ------------------------------------------------------------------------- */
 /** See @ref wlmtk_element_vmt_t::pointer_button. */
 bool _wlmtk_resizebar_area_element_pointer_button(
@@ -316,9 +294,10 @@ void test_area(bs_test_t *test_ptr)
     wlmtk_fake_window_t *fake_window_ptr = wlmtk_fake_window_create();
 
     wlmtk_resizebar_area_t *area_ptr = wlmtk_resizebar_area_create(
-        fake_window_ptr->window_ptr, NULL, WLR_EDGE_BOTTOM);
+        fake_window_ptr->window_ptr, WLR_EDGE_BOTTOM);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, area_ptr);
     wlmtk_element_t *element_ptr = wlmtk_resizebar_area_element(area_ptr);
+    wlmtk_element_set_visible(element_ptr, true);
 
     // Draw and verify release state.
     wlmtk_resizebar_style_t style = { .height = 7, .bezel_width = 1.0 };
@@ -335,9 +314,10 @@ void test_area(bs_test_t *test_ptr)
     BS_TEST_VERIFY_FALSE(test_ptr, fake_window_ptr->request_resize_called);
 
     // Pointer must be inside the button for accepting DOWN.
+    wlmtk_pointer_motion_event_t mev = { .x = 1, .y = 1 };
     BS_TEST_VERIFY_TRUE(
         test_ptr,
-        wlmtk_element_pointer_motion(element_ptr, 1, 1, 0));
+        wlmtk_element_pointer_motion(element_ptr, &mev));
     // Button down: pressed.
     wlmtk_button_event_t button = {
         .button = BTN_LEFT, .type = WLMTK_BUTTON_DOWN
