@@ -20,23 +20,29 @@
 
 #include "surface.h"
 
-#include "element.h"
-#include "gfxbuf.h"
-#include "util.h"
-
-#include <wlr/version.h>
+#include <libbase/libbase.h>
+#include <stdlib.h>
+#include <wayland-server-protocol.h>
+#include <wayland-util.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/types/wlr_compositor.h>
+#include <wlr/types/wlr_keyboard.h>
 #include <wlr/types/wlr_scene.h>
 #include <wlr/types/wlr_seat.h>
+#include <wlr/version.h>
 #undef WLR_USE_UNSTABLE
 
+#include "element.h"
+#include "container.h"
+#include "gfxbuf.h"  // IWYU pragma: keep
+#include "input.h"
+#include "util.h"
+
 /* == Declarations ========================================================= */
 
 static bool _wlmtk_surface_init(
     wlmtk_surface_t *surface_ptr,
-    struct wlr_surface *wlr_surface_ptr,
-    wlmtk_env_t *env_ptr);
+    struct wlr_surface *wlr_surface_ptr);
 static void _wlmtk_surface_fini(wlmtk_surface_t *surface_ptr);
 
 static void _wlmtk_surface_element_destroy(wlmtk_element_t *element_ptr);
@@ -49,18 +55,9 @@ static void _wlmtk_surface_element_get_dimensions(
     int *top_ptr,
     int *right_ptr,
     int *bottom_ptr);
-static void _wlmtk_surface_element_get_pointer_area(
+static bool _wlmtk_surface_element_pointer_accepts_motion(
     wlmtk_element_t *element_ptr,
-    int *left_ptr,
-    int *top_ptr,
-    int *right_ptr,
-    int *bottom_ptr);
-static void _wlmtk_surface_element_pointer_leave(wlmtk_element_t *element_ptr);
-static bool _wlmtk_surface_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec);
+    wlmtk_pointer_motion_event_t *motion_event_ptr);
 static bool _wlmtk_surface_element_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
@@ -69,10 +66,7 @@ static bool _wlmtk_surface_element_pointer_axis(
     struct wlr_pointer_axis_event *wlr_pointer_axis_event_ptr);
 static bool _wlmtk_surface_element_keyboard_event(
     wlmtk_element_t *element_ptr,
-    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    const xkb_keysym_t *key_syms,
-    size_t key_syms_count,
-    uint32_t modifiers);
+    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr);
 
 static void _wlmtk_surface_handle_wlr_scene_tree_node_destroy(
     struct wl_listener *listener_ptr,
@@ -86,6 +80,12 @@ static void _wlmtk_surface_handle_surface_map(
 static void _wlmtk_surface_handle_surface_unmap(
     struct wl_listener *listener_ptr,
     void *data_ptr);
+static void _wlmtk_surface_handle_element_pointer_leave(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmtk_surface_handle_element_pointer_motion(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 static void _wlmtk_surface_commit_size(
     wlmtk_surface_t *surface_ptr,
@@ -99,9 +99,7 @@ static const wlmtk_element_vmt_t surface_element_vmt = {
     .destroy = _wlmtk_surface_element_destroy,
     .create_scene_node = _wlmtk_surface_element_create_scene_node,
     .get_dimensions = _wlmtk_surface_element_get_dimensions,
-    .get_pointer_area = _wlmtk_surface_element_get_pointer_area,
-    .pointer_leave = _wlmtk_surface_element_pointer_leave,
-    .pointer_motion = _wlmtk_surface_element_pointer_motion,
+    .pointer_accepts_motion = _wlmtk_surface_element_pointer_accepts_motion,
     .pointer_button = _wlmtk_surface_element_pointer_button,
     .pointer_axis = _wlmtk_surface_element_pointer_axis,
     .keyboard_event = _wlmtk_surface_element_keyboard_event,
@@ -112,15 +110,16 @@ static const wlmtk_element_vmt_t surface_element_vmt = {
 /* ------------------------------------------------------------------------- */
 wlmtk_surface_t *wlmtk_surface_create(
     struct wlr_surface *wlr_surface_ptr,
-    wlmtk_env_t *env_ptr)
+    struct wlr_seat *wlr_seat_ptr)
 {
     wlmtk_surface_t *surface_ptr = logged_calloc(1, sizeof(wlmtk_surface_t));
     if (NULL == surface_ptr) return NULL;
 
-    if (!_wlmtk_surface_init(surface_ptr, wlr_surface_ptr, env_ptr)) {
+    if (!_wlmtk_surface_init(surface_ptr, wlr_surface_ptr)) {
         wlmtk_surface_destroy(surface_ptr);
         return NULL;
     }
+    surface_ptr->wlr_seat_ptr = wlr_seat_ptr;
 
     return surface_ptr;
 }
@@ -155,32 +154,30 @@ void wlmtk_surface_set_activated(
 {
     if (surface_ptr->activated == activated) return;
 
-    struct wlr_seat *wlr_seat_ptr = wlmtk_env_wlr_seat(surface_ptr->env_ptr);
-    struct wlr_keyboard *wlr_keyboard_ptr = wlr_seat_get_keyboard(wlr_seat_ptr);
-    if (activated) {
-        if (NULL != wlr_keyboard_ptr) {
-            wlr_seat_keyboard_notify_enter(
-                wlr_seat_ptr,
-                surface_ptr->wlr_surface_ptr,
-                wlr_keyboard_ptr->keycodes,
-                wlr_keyboard_ptr->num_keycodes,
-                &wlr_keyboard_ptr->modifiers);
-
-        }
-        if (NULL != surface_ptr->super_element.parent_container_ptr) {
-            wlmtk_container_set_keyboard_focus_element(
-                surface_ptr->super_element.parent_container_ptr,
-                &surface_ptr->super_element);
-        }
-    } else {
-        if (wlr_seat_ptr->keyboard_state.focused_surface ==
-            surface_ptr->wlr_surface_ptr) {
-            wlr_seat_keyboard_clear_focus(wlr_seat_ptr);
-            wlmtk_container_set_keyboard_focus_element(
-                surface_ptr->super_element.parent_container_ptr, NULL);
+    if (NULL != surface_ptr->wlr_seat_ptr) {
+        struct wlr_keyboard *wlr_keyboard_ptr = wlr_seat_get_keyboard(
+            surface_ptr->wlr_seat_ptr);
+        if (activated) {
+            if (NULL != wlr_keyboard_ptr) {
+                wlr_seat_keyboard_notify_enter(
+                    surface_ptr->wlr_seat_ptr,
+                    surface_ptr->wlr_surface_ptr,
+                    wlr_keyboard_ptr->keycodes,
+                    wlr_keyboard_ptr->num_keycodes,
+                    &wlr_keyboard_ptr->modifiers);
+            }
+        } else if (surface_ptr->wlr_seat_ptr->keyboard_state.focused_surface ==
+                   surface_ptr->wlr_surface_ptr) {
+            wlr_seat_keyboard_clear_focus(surface_ptr->wlr_seat_ptr);
         }
     }
 
+    if (NULL != surface_ptr->super_element.parent_container_ptr) {
+        wlmtk_container_set_keyboard_focus_element(
+            surface_ptr->super_element.parent_container_ptr,
+            &surface_ptr->super_element,
+            activated);
+    }
     surface_ptr->activated = activated;
 }
 
@@ -218,28 +215,34 @@ void wlmtk_surface_connect_unmap_listener_signal(
  *
  * @param surface_ptr
  * @param wlr_surface_ptr
- * @param env_ptr
  *
  * @return true on success.
  */
 bool _wlmtk_surface_init(
     wlmtk_surface_t *surface_ptr,
-    struct wlr_surface *wlr_surface_ptr,
-    wlmtk_env_t *env_ptr)
+    struct wlr_surface *wlr_surface_ptr)
 {
     BS_ASSERT(NULL != surface_ptr);
-    memset(surface_ptr, 0, sizeof(wlmtk_surface_t));
+    *surface_ptr = (wlmtk_surface_t){};
 
-    if (!wlmtk_element_init(&surface_ptr->super_element, env_ptr)) {
+    if (!wlmtk_element_init(&surface_ptr->super_element)) {
         _wlmtk_surface_fini(surface_ptr);
         return false;
     }
     surface_ptr->orig_super_element_vmt = wlmtk_element_extend(
         &surface_ptr->super_element, &surface_element_vmt);
-    surface_ptr->env_ptr = env_ptr;
+    wlmtk_util_connect_listener_signal(
+        &surface_ptr->super_element.events.pointer_leave,
+        &surface_ptr->element_pointer_leave_listener,
+        _wlmtk_surface_handle_element_pointer_leave);
+    wlmtk_util_connect_listener_signal(
+        &surface_ptr->super_element.events.pointer_motion,
+        &surface_ptr->element_pointer_motion_listener,
+        _wlmtk_surface_handle_element_pointer_motion);
 
     surface_ptr->wlr_surface_ptr = wlr_surface_ptr;
     if (NULL != surface_ptr->wlr_surface_ptr) {
+        surface_ptr->wlr_surface_ptr->data = surface_ptr;
         wlmtk_util_connect_listener_signal(
             &wlr_surface_ptr->events.commit,
             &surface_ptr->surface_commit_listener,
@@ -276,8 +279,12 @@ void _wlmtk_surface_fini(wlmtk_surface_t *surface_ptr)
         wlmtk_util_disconnect_listener(&surface_ptr->surface_unmap_listener);
     }
 
+    wlmtk_util_disconnect_listener(
+        &surface_ptr->element_pointer_motion_listener);
+    wlmtk_util_disconnect_listener(
+        &surface_ptr->element_pointer_leave_listener);
     wlmtk_element_fini(&surface_ptr->super_element);
-    memset(surface_ptr, 0, sizeof(wlmtk_surface_t));
+    *surface_ptr = (wlmtk_surface_t){};
 }
 
 /* ------------------------------------------------------------------------- */
@@ -346,94 +353,21 @@ void _wlmtk_surface_element_get_dimensions(
 
 /* ------------------------------------------------------------------------- */
 /**
- * Overwrites the element's get_pointer_area method: Returns the extents of
- * the surface and all subsurfaces.
+ * Implements @ref wlmtk_element_vmt_t::pointer_accepts_motion.
  *
  * @param element_ptr
- * @param left_ptr            Leftmost position. May be NULL.
- * @param top_ptr             Topmost position. May be NULL.
- * @param right_ptr           Rightmost position. Ma be NULL.
- * @param bottom_ptr          Bottommost position. May be NULL.
+ * @param motion_event_ptr
+ *
+ * @return true if there is a surface (or sub-surface) at the given coordinates
+ * that will accept the pointer's movement.
  */
-void _wlmtk_surface_element_get_pointer_area(
+bool _wlmtk_surface_element_pointer_accepts_motion(
     wlmtk_element_t *element_ptr,
-    int *left_ptr,
-    int *top_ptr,
-    int *right_ptr,
-    int *bottom_ptr)
+    wlmtk_pointer_motion_event_t *motion_event_ptr)
 {
     wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_surface_t, super_element);
 
-    struct wlr_box box;
-    if (NULL == surface_ptr->wlr_surface_ptr) {
-        box.x = 0;
-        box.y = 0;
-        box.width = surface_ptr->committed_width;
-        box.height = surface_ptr->committed_height;
-    } else {
-        wlr_surface_get_extends(surface_ptr->wlr_surface_ptr, &box);
-    }
-
-    if (NULL != left_ptr) *left_ptr = box.x;
-    if (NULL != top_ptr) *top_ptr = box.y;
-    if (NULL != right_ptr) *right_ptr = box.width - box.x;
-    if (NULL != bottom_ptr) *bottom_ptr = box.height - box.y;
-}
-
-/* ------------------------------------------------------------------------- */
-/**
- * Implements the element's leave method: If there's a WLR (sub)surface
- * currently holding focus, that will be cleared.
- *
- * @param element_ptr
- */
-void _wlmtk_surface_element_pointer_leave(wlmtk_element_t *element_ptr)
-{
-    wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_surface_t, super_element);
-
-    // If the current surface's parent is our surface: clear it.
-    struct wlr_surface *focused_wlr_surface_ptr =
-        wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr
-            )->pointer_state.focused_surface;
-    if (NULL != focused_wlr_surface_ptr &&
-        wlr_surface_get_root_surface(focused_wlr_surface_ptr) ==
-        surface_ptr->wlr_surface_ptr) {
-        wlr_seat_pointer_clear_focus(
-            wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr));
-    }
-}
-
-/* ------------------------------------------------------------------------- */
-/**
- * Pass pointer motion events to client's surface.
- *
- * Identifies the surface (or sub-surface) at the given coordinates, and pass
- * on the motion event to that surface. If needed, will update the seat's
- * pointer focus.
- *
- * @param element_ptr
- * @param x                   Pointer horizontal position, relative to this
- *                            element's node.
- * @param y                   Pointer vertical position, relative to this
- *                            element's node.
- * @param time_msec
- *
- * @return Whether if the motion is within the area.
- */
-bool _wlmtk_surface_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec)
-{
-    wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_surface_t, super_element);
-
-    surface_ptr->orig_super_element_vmt.pointer_motion(
-        element_ptr, x, y, time_msec);
-
     if (NULL == surface_ptr->super_element.wlr_scene_node_ptr) return false;
 
     // Get the layout local coordinates of the node, so we can adjust the
@@ -447,32 +381,13 @@ bool _wlmtk_surface_element_pointer_motion(
     double node_x, node_y;
     struct wlr_scene_node *wlr_scene_node_ptr = wlr_scene_node_at(
         surface_ptr->super_element.wlr_scene_node_ptr,
-        x + lx, y + ly, &node_x, &node_y);
+        motion_event_ptr->x + lx,
+        motion_event_ptr->y + ly,
+        &node_x,
+        &node_y);
 
-    if (NULL == wlr_scene_node_ptr ||
-        WLR_SCENE_NODE_BUFFER != wlr_scene_node_ptr->type) {
-        return false;
-    }
-
-    struct wlr_scene_buffer *wlr_scene_buffer_ptr =
-        wlr_scene_buffer_from_node(wlr_scene_node_ptr);
-    struct wlr_scene_surface *wlr_scene_surface_ptr =
-        wlr_scene_surface_try_from_buffer(wlr_scene_buffer_ptr);
-    if (NULL == wlr_scene_surface_ptr) {
-        return true;
-    }
-
-    BS_ASSERT(surface_ptr->wlr_surface_ptr ==
-              wlr_surface_get_root_surface(wlr_scene_surface_ptr->surface));
-    wlr_seat_pointer_notify_enter(
-        wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr),
-        wlr_scene_surface_ptr->surface,
-        node_x, node_y);
-    wlr_seat_pointer_notify_motion(
-        wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr),
-        time_msec,
-        node_x, node_y);
-    return true;
+    return (NULL != wlr_scene_node_ptr &&
+            WLR_SCENE_NODE_BUFFER == wlr_scene_node_ptr->type);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -496,8 +411,7 @@ bool _wlmtk_surface_element_pointer_button(
 
     // Complain if the surface isn't part of our responsibility.
     struct wlr_surface *focused_wlr_surface_ptr =
-        wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr
-            )->pointer_state.focused_surface;
+        surface_ptr->wlr_seat_ptr->pointer_state.focused_surface;
     if (NULL == focused_wlr_surface_ptr) return false;
     // TODO(kaeser@gubbe.ch): Dragging the pointer from an activated window
     // over to a non-activated window will trigger the condition here on the
@@ -522,7 +436,7 @@ bool _wlmtk_surface_element_pointer_button(
             WLR_BUTTON_PRESSED : WLR_BUTTON_RELEASED;
 #endif // WLR_VERSION_NUM >= (18 << 8)
         wlr_seat_pointer_notify_button(
-            wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr),
+            surface_ptr->wlr_seat_ptr,
             button_event_ptr->time_msec,
             button_event_ptr->button,
             state);
@@ -551,12 +465,11 @@ bool _wlmtk_surface_element_pointer_axis(
 
     // Complain if the surface isn't part of our responsibility.
     struct wlr_surface *focused_wlr_surface_ptr =
-        wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr
-            )->pointer_state.focused_surface;
+        surface_ptr->wlr_seat_ptr->pointer_state.focused_surface;
     if (NULL == focused_wlr_surface_ptr) return false;
 
     wlr_seat_pointer_notify_axis(
-        wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr),
+        surface_ptr->wlr_seat_ptr,
         wlr_pointer_axis_event_ptr->time_msec,
         wlr_pointer_axis_event_ptr->orientation,
         wlr_pointer_axis_event_ptr->delta,
@@ -577,40 +490,23 @@ bool _wlmtk_surface_element_pointer_axis(
  *
  * @param element_ptr
  * @param wlr_keyboard_key_event_ptr
- * @param key_syms
- * @param key_syms_count
- * @param modifiers
  *
  * @return true if the axis event was handled.
  */
 bool _wlmtk_surface_element_keyboard_event(
     wlmtk_element_t *element_ptr,
-    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    __UNUSED__ const xkb_keysym_t *key_syms,
-    __UNUSED__ size_t key_syms_count,
-    __UNUSED__ uint32_t modifiers)
+    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr)
 {
     wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_surface_t, super_element);
 
-    struct wlr_seat *wlr_seat_ptr = wlmtk_env_wlr_seat(element_ptr->env_ptr);
-    struct wlr_keyboard *wlr_keyboard_ptr = wlr_seat_get_keyboard(
-        wlr_seat_ptr);
+    if (!surface_ptr->activated) return false;
 
-    if (NULL == wlr_keyboard_ptr) return false;
+    // Guard clauses.
+    if (NULL == surface_ptr->wlr_seat_ptr) return false;
 
-    wlr_seat_keyboard_notify_enter(
-        wlr_seat_ptr,
-        surface_ptr->wlr_surface_ptr,
-        wlr_keyboard_ptr->keycodes,
-        wlr_keyboard_ptr->num_keycodes,
-        &wlr_keyboard_ptr->modifiers);
-
-    wlr_seat_set_keyboard(
-        wlr_seat_ptr,
-        wlr_keyboard_ptr);
     wlr_seat_keyboard_notify_key(
-        wlr_seat_ptr,
+        surface_ptr->wlr_seat_ptr,
         wlr_keyboard_key_event_ptr->time_msec,
         wlr_keyboard_key_event_ptr->keycode,
         wlr_keyboard_key_event_ptr->state);
@@ -682,6 +578,95 @@ void _wlmtk_surface_handle_surface_unmap(
     wlmtk_element_set_visible( wlmtk_surface_element(surface_ptr), false);
 }
 
+/* ------------------------------------------------------------------------- */
+/**
+ * Handles pointer leave: If there's a WLR (sub)surface currently holding
+ * focus, that will be cleared.
+ *
+ * @param listener_ptr
+ * @param data_ptr
+ */
+void _wlmtk_surface_handle_element_pointer_leave(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_surface_t, element_pointer_leave_listener);
+
+    // Guard clause.
+    if (NULL == surface_ptr->wlr_seat_ptr) return;
+
+    // If the current surface's parent is our surface: clear it.
+    struct wlr_surface *focused_wlr_surface_ptr =
+        surface_ptr->wlr_seat_ptr->pointer_state.focused_surface;
+    if (NULL != focused_wlr_surface_ptr &&
+        wlr_surface_get_root_surface(focused_wlr_surface_ptr) ==
+        surface_ptr->wlr_surface_ptr) {
+        wlr_seat_pointer_clear_focus(surface_ptr->wlr_seat_ptr);
+    }
+}
+
+/* ------------------------------------------------------------------------- */
+/**
+ * Handles pointer motion: Passes the motion events to client's surface(s).
+ *
+ * Identifies the surface (or sub-surface) at the given coordinates, and pass
+ * on the motion event to that surface. If needed, will update the seat's
+ * pointer focus.
+ *
+ * @param listener_ptr
+ * @param data_ptr            Points to a @ref wlmtk_pointer_motion_event_t.
+ */
+void _wlmtk_surface_handle_element_pointer_motion(
+    struct wl_listener *listener_ptr,
+    void *data_ptr)
+{
+    wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_surface_t, element_pointer_motion_listener);
+    wlmtk_pointer_motion_event_t *motion_event_ptr = data_ptr;
+
+    if (NULL == surface_ptr->super_element.wlr_scene_node_ptr) return;
+
+    // Get the layout local coordinates of the node, so we can adjust the
+    // node-local (x, y) for the `wlr_scene_node_at` call.
+    int lx, ly;
+    if (!wlr_scene_node_coords(
+            surface_ptr->super_element.wlr_scene_node_ptr, &lx, &ly)) {
+        return;
+    }
+    // Get the node below the cursor. Return if there's no buffer node.
+    double node_x, node_y;
+    struct wlr_scene_node *wlr_scene_node_ptr = wlr_scene_node_at(
+        surface_ptr->super_element.wlr_scene_node_ptr,
+        motion_event_ptr->x + lx,
+        motion_event_ptr->y + ly,
+        &node_x,
+        &node_y);
+
+    if (NULL == wlr_scene_node_ptr ||
+        WLR_SCENE_NODE_BUFFER != wlr_scene_node_ptr->type) {
+        return;
+    }
+
+    struct wlr_scene_buffer *wlr_scene_buffer_ptr =
+        wlr_scene_buffer_from_node(wlr_scene_node_ptr);
+    struct wlr_scene_surface *wlr_scene_surface_ptr =
+        wlr_scene_surface_try_from_buffer(wlr_scene_buffer_ptr);
+    if (NULL == wlr_scene_surface_ptr) return;
+
+    BS_ASSERT(surface_ptr->wlr_surface_ptr ==
+              wlr_surface_get_root_surface(wlr_scene_surface_ptr->surface));
+    if (NULL != surface_ptr->wlr_seat_ptr) {
+        wlr_seat_pointer_notify_enter(
+            surface_ptr->wlr_seat_ptr,
+            wlr_scene_surface_ptr->surface,
+            node_x, node_y);
+        wlr_seat_pointer_notify_motion(
+            surface_ptr->wlr_seat_ptr,
+            motion_event_ptr->time_msec,
+            node_x, node_y);
+    }
+}
 
 /* ------------------------------------------------------------------------- */
 /**
@@ -703,7 +688,7 @@ void _wlmtk_surface_commit_size(
     }
 
     if (NULL != surface_ptr->super_element.parent_container_ptr) {
-        wlmtk_container_update_layout(
+        wlmtk_container_update_layout_and_pointer_focus(
             surface_ptr->super_element.parent_container_ptr);
     }
 }
@@ -716,23 +701,19 @@ static void _wlmtk_fake_surface_element_destroy(
 static struct wlr_scene_node *_wlmtk_fake_surface_element_create_scene_node(
     wlmtk_element_t *element_ptr,
     struct wlr_scene_tree *wlr_scene_tree_ptr);
-static bool _wlmtk_fake_surface_element_pointer_motion(
+static bool _wlmtk_fake_surface_element_pointer_accepts_motion(
     wlmtk_element_t *element_ptr,
-    double x, double y,
-    __UNUSED__ uint32_t time_msec);
+    wlmtk_pointer_motion_event_t *motion_event_ptr);
 static bool _wlmtk_fake_surface_element_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
-static void _wlmtk_fake_surface_element_pointer_leave(
-    wlmtk_element_t *element_ptr);
 
 /** Extensions to the surface's super elements virtual methods. */
 static const wlmtk_element_vmt_t _wlmtk_fake_surface_element_vmt = {
     .destroy = _wlmtk_fake_surface_element_destroy,
     .create_scene_node = _wlmtk_fake_surface_element_create_scene_node,
-    .pointer_motion = _wlmtk_fake_surface_element_pointer_motion,
+    .pointer_accepts_motion = _wlmtk_fake_surface_element_pointer_accepts_motion,
     .pointer_button = _wlmtk_fake_surface_element_pointer_button,
-    .pointer_leave = _wlmtk_fake_surface_element_pointer_leave,
 };
 
 /* ------------------------------------------------------------------------- */
@@ -742,7 +723,7 @@ wlmtk_fake_surface_t *wlmtk_fake_surface_create(void)
         1, sizeof(wlmtk_fake_surface_t));
     if (NULL == fake_surface_ptr) return NULL;
 
-    _wlmtk_surface_init(&fake_surface_ptr->surface, NULL, NULL);
+    _wlmtk_surface_init(&fake_surface_ptr->surface, NULL);
     wlmtk_element_extend(
         &fake_surface_ptr->surface.super_element,
         &_wlmtk_fake_surface_element_vmt);
@@ -752,7 +733,7 @@ wlmtk_fake_surface_t *wlmtk_fake_surface_create(void)
 /* ------------------------------------------------------------------------- */
 wlmtk_surface_t *wlmtk_fake_surface_create_inject(
     __UNUSED__ struct wlr_surface *wlr_surface_ptr,
-    __UNUSED__ wlmtk_env_t *env_ptr)
+    __UNUSED__ struct wlr_seat *wlr_seat_ptr)
 {
     wlmtk_fake_surface_t *fake_surface_ptr = wlmtk_fake_surface_create();
     if (NULL == fake_surface_ptr) return NULL;
@@ -806,17 +787,18 @@ struct wlr_scene_node *_wlmtk_fake_surface_element_create_scene_node(
 }
 
 /* ------------------------------------------------------------------------- */
-/** Fake for @ref wlmtk_element_vmt_t::pointer_motion. True if in committed. */
-bool _wlmtk_fake_surface_element_pointer_motion(
+/** Fake for @ref wlmtk_element_vmt_t::pointer_accepts_motion. True if in. */
+bool _wlmtk_fake_surface_element_pointer_accepts_motion(
     wlmtk_element_t *element_ptr,
-    double x, double y,
-    __UNUSED__ uint32_t time_msec)
+    wlmtk_pointer_motion_event_t *motion_event_ptr)
 {
     wlmtk_fake_surface_t *fake_surface_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_fake_surface_t, surface.super_element);
 
-    return (0 <= x && x < fake_surface_ptr->surface.committed_width &&
-            0 <= y && y < fake_surface_ptr->surface.committed_height);
+    return (0 <= motion_event_ptr->x &&
+            motion_event_ptr->x < fake_surface_ptr->surface.committed_width &&
+            0 <= motion_event_ptr->y &&
+            motion_event_ptr->y < fake_surface_ptr->surface.committed_height);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -828,14 +810,6 @@ bool _wlmtk_fake_surface_element_pointer_button(
     return true;
 }
 
-/* ------------------------------------------------------------------------- */
-/** Fake for @ref wlmtk_element_vmt_t::pointer_leave. Does nothing. */
-void _wlmtk_fake_surface_element_pointer_leave(
-    __UNUSED__ wlmtk_element_t *element_ptr)
-{
-    // Nothing to do.
-}
-
 /* == Unit tests =========================================================== */
 
 static void test_create_destroy(bs_test_t *test_ptr);
