@@ -20,24 +20,32 @@
 #ifndef __XDG_POPUP_H__
 #define __XDG_POPUP_H__
 
-#include "toolkit/toolkit.h"
-
+#include <wayland-server-core.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/types/wlr_xdg_shell.h>
 #undef WLR_USE_UNSTABLE
 
+#include "toolkit/toolkit.h"
+
+struct wlr_seat;
+struct wlr_xdg_popup;
+
+struct _wlmaker_xdg_popup_t;
+/** Forward declaration: State of the toolkit's XDG popup. */
+typedef struct _wlmaker_xdg_popup_t wlmaker_xdg_popup_t;
+
 #ifdef __cplusplus
 extern "C" {
 #endif  // __cplusplus
 
-/** Forward declaration: State of the toolkit's XDG popup. */
-typedef struct _wlmaker_xdg_popup_t wlmaker_xdg_popup_t;
-
 /** State of toolkit popup. */
 struct _wlmaker_xdg_popup_t {
     /** Super class: popup. */
     wlmtk_popup_t             super_popup;
 
+    /** Seat. */
+    struct wlr_seat           *wlr_seat_ptr;
+
     /** Surface of the popup. */
     wlmtk_surface_t           *surface_ptr;
     /** The WLR popup. */
@@ -60,7 +68,7 @@ struct _wlmaker_xdg_popup_t {
  */
 wlmaker_xdg_popup_t *wlmaker_xdg_popup_create(
     struct wlr_xdg_popup *wlr_xdg_popup_ptr,
-    wlmtk_env_t *env_ptr);
+    struct wlr_seat *wlr_seat_ptr);
 
 /**
  * Destroys the popup.
